# -*- coding: utf-8 -*-


from __future__ import absolute_import
from dp_tornado.engine.helper import Helper as dpHelper

import re


class NumericHelper(dpHelper):
    def extract_numbers(self, string):
        if self.helper.system.py_version <= 2:
            types = basestring,
            types_num = (int, long)
        else:
            types = str,
            types_num = (int, )

        if isinstance(string, types_num):
            return str(string)

        if not isinstance(string, types):
            return None

        return re.sub(r'\D+', '', string)

    def number_format(self, value, tsep=',', dsep='.'):
        value = self.extract_numbers(value)

        if not value:
            return '0'

        s = str(value)
        cnt = 0
        numchars = dsep + '0123456789'
        ls = len(s)
        while cnt < ls and s[cnt] not in numchars:
            cnt += 1

        lhs = s[:cnt]
        s = s[cnt:]
        if not dsep:
            cnt = -1
        else:
            cnt = s.rfind(dsep)
        if cnt > 0:
            rhs = dsep + s[cnt+1:]
            s = s[:cnt]
        else:
            rhs = ''

        splt = ''
        while s != '':
            splt = s[-3:] + tsep + splt
            s = s[:-3]

        return lhs + splt[:-1] + rhs

    def int(self, a):
        if self.helper.system.py_version <= 2:
            return int(a) if a else 0
        else:
            return int(a) if a else 0

    def long(self, a):
        if self.helper.system.py_version <= 2:
            return long(a) if a else long(0)
        else:
            return int(a) if a else 0

    @property
    def xxx(self):
        return 'ABtQR5JxfghijN_qrb4KSy9-Uuvw6ZVcLnpMFGkeTPd3WXYEHCDma78sz12'

    def to_xxx(self, x):
        __ = self.xxx
        _ = ''

        while (True):
            if x <= 0:
                break

            k = x % len(__)
            _ = '%s%s' % (__[k:k+1], _)
            x = self.long(x / len(__))

        return _

    def from_xxx(self, x):
        __ = self.xxx
        _ = 0
        c = 0

        for i in reversed(str(x)):
            i = __.find(i)
            _ += i * (len(__) ** c)
            c += 1

        return _



# -*- coding: utf-8 -*-


from .singleton import Singleton as dpSingleton
from .engine import Engine as dpEngine

engine = dpEngine()


class View(dpSingleton):
    @staticmethod
    def render(controller, template_name, kwargs=None):
        if kwargs:
            controller.parent.render(template_name, **kwargs)
        else:
            controller.parent.render(template_name)

    @staticmethod
    def render_string(controller, template_name, kwargs=None):
        if engine.helper.system.py_version <= 2:
            if kwargs:
                return engine.helper.string.to_str(controller.parent.render_string(template_name, **kwargs))
            else:
                return engine.helper.string.to_str(controller.parent.render_string(template_name))
        else:
            if kwargs:
                return str(controller.parent.render_string(template_name, **kwargs), 'UTF-8')
            else:
                return str(controller.parent.render_string(template_name), 'UTF-8')

    @staticmethod
    def write(controller, chunk):
        controller.parent.write(chunk)

    @staticmethod
    def finish(controller, chunk=None):
        controller.parent.finish(chunk)


# -*- coding: utf-8 -*-


import os
import time
import threading
import subprocess
import tornado.options

from dp_tornado.engine.scheduler import tornado_subprocess
from dp_tornado.engine.engine import Engine


try:
    from croniter import croniter
except ImportError:
    croniter = None


class Scheduler(threading.Thread, Engine):
    def __init__(self, schedules):
        self.interrupted = False
        self.schedules = []
        self.path = os.path.dirname(os.path.realpath(__file__))
        self.path = os.path.join(self.path, 'runner.py')
        self.python = tornado.options.options.python
        self.application_path = tornado.options.options.application_path
        self.ts = self.helper.datetime.time()
        self.reference_count = 0

        for e in schedules:
            i = e[2] if len(e) >= 3 and isinstance(e[2], int) else 1

            for i in range(i):
                s = e[0] if isinstance(e[0], int) else croniter(e[0], self.ts)

                self.schedules.append({
                    'c': e[1],
                    's': s,
                    'n': self.ts + 5 if isinstance(e[0], int) else s.get_next()
                })

        threading.Thread.__init__(self)

    def run(self):
        if not self.schedules:
            return

        while not self.interrupted:
            ts = self.helper.datetime.time()

            for e in self.schedules:
                if ts >= e['n']:
                    try:
                        e['n'] = ts + e['s'] if isinstance(e['s'], int) else e['s'].get_next()
                        args = [self.python, self.path, self.application_path, e['c']]
                        self.reference_count += 1

                        if subprocess.mswindows:
                            subprocess.Popen(
                                ' '.join(args), shell=True, close_fds=False if subprocess.mswindows else True)
                        else:
                            h = SchedulerHandler()
                            h.attach(args=args, timeout=0, ref=self.reference_count)

                    except Exception as e:
                        self.logging.exception(e)

            time.sleep(2)


class SchedulerHandler(Engine):
    args = None
    ref = 0

    def on_done(self, status, stdout, stderr, has_timed_out):
        if has_timed_out:
            self.logging.error('Scheduler done with timed out [%s] (%s)' % (' '.join(self.args[2:]), self.ref))

            if stdout:
                self.logging.error(stdout)
            if stderr:
                self.logging.error(stderr)

            return

        if stdout:
            self.logging.info('Scheduler done with stdout [%s] (%s)' % (' '.join(self.args[2:]), self.ref))
            self.logging.info(stdout)
            return

        if stderr:
            self.logging.error('Scheduler done with stderr [%s] (%s)' % (' '.join(self.args[2:]), self.ref))
            self.logging.error(stderr)
            return

        self.logging.info('Scheduler done [%s] (%s)' % (' '.join(self.args[2:]), self.ref))

    def attach(self, args, timeout=0, ref=None):
        self.args = args
        self.ref = ref
        self.logging.info('Scheduler attach [%s] (%s)' % (' '.join(self.args[2:]), self.ref))

        tornado_subprocess.Subprocess(
            callback=self.on_done,
            timeout=timeout or 3600*24*7,
            args=self.args).start()


class Processor(Engine):
    def run(self):
        raise NotImplementedError
